/////////////////////////////////////////////////////////////////////////////
//
//  WinAPI_23.cpp
//
//  (C) Radek Chalupa, 2002
//
/////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <stdio.h>
#include <tchar.h>

#define APP_NAME    TEXT("Registry demo")
#define CLASS_NAME  TEXT("WinAPIDemoClass")

HINSTANCE g_hInst;
HWND g_hWnd;

TCHAR chInfoText[150];
DWORD dwWindowPos;
DWORD dwWindowSize;


/////////////////////////////////////////////////////////////////////////////
// Funkce pro prci s registry

#define REGISTRY_KEY_NAME "Software\\Radek Chalupa\\Ucime se WinAPI"

HKEY registry_OpenKey(LPCTSTR lpKeyName, LPBOOL lpbCreated)
{
	DWORD dwDisp = 0;
	HKEY hkResult = NULL;
	if ( RegCreateKeyEx(HKEY_CURRENT_USER, lpKeyName, 0, NULL, REG_OPTION_NON_VOLATILE,
		KEY_ALL_ACCESS, NULL, &hkResult, &dwDisp) != ERROR_SUCCESS )
		return NULL;
	if ( lpbCreated )
		*lpbCreated = ( dwDisp & REG_CREATED_NEW_KEY );
	return hkResult;
}

BOOL registry_CloseKey(HKEY hKey)
{
	return ( RegCloseKey(hKey) == ERROR_SUCCESS);
}

BOOL registry_KeyExists(HKEY hKey, LPCTSTR lpKeyName)
{
	return ( RegQueryValueEx(hKey, lpKeyName, NULL, NULL, NULL, NULL) == ERROR_SUCCESS );
}

BOOL registry_ReadDWORD(HKEY hKey, LPCTSTR lpValueName, LPDWORD lpdwResult)
{
	if ( !registry_KeyExists(hKey, lpValueName) )
		return FALSE;
	DWORD dwValue, dwSize, dwType;
	dwSize = sizeof(DWORD);
	dwValue = 0;
	if ( RegQueryValueEx(hKey, lpValueName, NULL, &dwType, (LPBYTE)&dwValue, &dwSize)
		!= ERROR_SUCCESS )
		return FALSE;
	*lpdwResult = dwValue;
	return TRUE;
}

BOOL registry_ReadString(HKEY hKey, LPCTSTR lpValueName, LPTSTR lpString)
{
	if ( !registry_KeyExists(hKey, lpValueName) )
		return FALSE;
	DWORD dwSize, dwType;
	dwSize = sizeof(chInfoText);
	if ( RegQueryValueEx(hKey, lpValueName, NULL, &dwType, (LPBYTE)lpString, &dwSize)
		!= ERROR_SUCCESS )
		return FALSE;
	return TRUE;
}

BOOL registry_WriteDWORD(HKEY hKey, LPCTSTR lpValueName, DWORD dwValue)
{
	return  ( RegSetValueEx(hKey, lpValueName, 0, REG_DWORD,
		(CONST BYTE*)&dwValue, sizeof(DWORD)) == ERROR_SUCCESS );
}

BOOL registry_WriteString(HKEY hKey, LPCTSTR lpValueName, LPCTSTR lpText)
{
	return ( RegSetValueEx(hKey, lpValueName, 0, REG_SZ, (CONST BYTE*)lpText,
		(lstrlen(lpText)+1)*sizeof(TCHAR)) ==	ERROR_SUCCESS );
}

/////////////////////////////////////////////////////////////////////////////
// Handler zprvy WM_PAINT

void OnPaint(HWND hWnd)
{
	PAINTSTRUCT ps;
	HDC hDC;
	RECT rect;
	GetClientRect(g_hWnd, &rect);
	hDC = BeginPaint(hWnd, &ps);
	SetBkMode(hDC, TRANSPARENT);
	DrawText(hDC, chInfoText, -1, &rect,
		DT_SINGLELINE | DT_CENTER | DT_VCENTER);
	EndPaint(hWnd, &ps);
}

/////////////////////////////////////////////////////////////////////////////

BOOL WriteRegistry()
{
	BOOL bRes = TRUE;
	HKEY hKey =  registry_OpenKey(REGISTRY_KEY_NAME, NULL);
	if ( !hKey )
		return FALSE;
	if ( !registry_WriteDWORD(hKey, TEXT("WindowPos"), dwWindowPos) )
		bRes = FALSE;
	if ( !registry_WriteDWORD(hKey, TEXT("WindowSize"), dwWindowSize) )
		bRes = FALSE;
	if ( !registry_WriteString(hKey, TEXT("Info"), chInfoText) )
		bRes = FALSE;
	if ( !registry_CloseKey(hKey) )
		bRes = FALSE;
	return bRes;
}

/////////////////////////////////////////////////////////////////////////////

void OnClose()
{
	SYSTEMTIME st;
	GetLocalTime(&st);
	sprintf(chInfoText, "Pedchoz ukonen: %d.%d.%d  %.2d:%.2d",
		st.wDay, st.wMonth, st.wYear, st.wHour, st.wMinute);
	RECT rect;
	GetWindowRect(g_hWnd, &rect);
	dwWindowPos = MAKELONG(rect.left, rect.top);
	dwWindowSize = MAKELONG(rect.right-rect.left, rect.bottom-rect.top);
	WriteRegistry();
}

/////////////////////////////////////////////////////////////////////////////
// Procedura okna 

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{
		case WM_CLOSE:
			OnClose();
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		case WM_PAINT:
			OnPaint(hWnd);
			break;
	}
	return DefWindowProc(hWnd, message, wParam, lParam);
}

/////////////////////////////////////////////////////////////////////////////

BOOL ReadRegistry()
{
	BOOL bRes = TRUE;
	BOOL bFirst = FALSE;
	HKEY hKey = registry_OpenKey(REGISTRY_KEY_NAME, &bFirst);
	if ( !hKey )
		return FALSE;
	if ( bFirst )
	{
		dwWindowPos =  0x00600080;
		dwWindowSize = 0x01500200;
		lstrcpy(chInfoText, TEXT("Prvn sputn programu"));
		return TRUE;
	}
	if ( !registry_ReadDWORD(hKey, "WindowPos", &dwWindowPos) )
		bRes = FALSE;
	if ( !registry_ReadDWORD(hKey, "WindowSize", &dwWindowSize) )
		bRes = FALSE;
	if ( !registry_ReadString(hKey, "Info", chInfoText) )
		bRes = FALSE;
	if ( !registry_CloseKey(hKey) )
		bRes = FALSE;
	return bRes;
}

/////////////////////////////////////////////////////////////////////////////
// Registrace tdy okna a vytvoen okna

BOOL InitApp()
{
	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX); 
	wcex.style = CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc = (WNDPROC)WndProc;
	wcex.cbClsExtra = 0;
	wcex.cbWndExtra = 0;
	wcex.hInstance = g_hInst;
	wcex.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wcex.hIconSm = LoadIcon(NULL, IDI_APPLICATION);
	wcex.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground = GetSysColorBrush(COLOR_3DFACE);
	wcex.lpszMenuName = NULL;
	wcex.lpszClassName = CLASS_NAME;
	if ( !RegisterClassEx(&wcex) )
		return FALSE;
	if ( !ReadRegistry() )
		return FALSE;
	g_hWnd = CreateWindowEx(0, CLASS_NAME, APP_NAME,
		WS_OVERLAPPEDWINDOW,
		LOWORD(dwWindowPos),
		HIWORD(dwWindowPos),
		LOWORD(dwWindowSize),
		HIWORD(dwWindowSize),
		NULL, NULL, g_hInst, NULL);
	if (!g_hWnd)
		return FALSE;
	ShowWindow(g_hWnd, SW_SHOW);
	UpdateWindow(g_hWnd);
}

/////////////////////////////////////////////////////////////////////////////
// Hlavn funkce programu pro Windows

int WINAPI _tWinMain(HINSTANCE hInstance,	HINSTANCE hPrevInstance,
	LPTSTR lpCmdLine, int nCmdShow)
{
	MSG msg;
	g_hInst = hInstance;
	if ( !InitApp() ) 
		return -1;
	// Hlavn smyka zprv:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return 0;
}
